#include "RuleListView.h"

#include "NameInputDialog.h"
#include "MessageConstants.h"

RuleItem	RuleListView::clip_board("empty",0);
BLocker	RuleListView::clip_board_lock("rule clip");
const char* RuleListView::DATA_HEADER="%LifeHex vers.1.0";

RuleListView::RuleListView(BRect frame)
	: BListView
		( frame
		, "rule list"
		, B_SINGLE_SELECTION_LIST
		, B_FOLLOW_ALL_SIDES
		)
	, is_dirty(false)
{	
}

RuleListView::~RuleListView()
{
	RemoveAll();
}



#define ITEM_INIT \
	if(index<0) index = CurrentSelection();\
	RuleItem* item = (RuleItem*)ItemAt(index);



bool
RuleListView::IsDirty() const
{
	return is_dirty;
}
void
RuleListView::SetDirty(bool d)
{
	is_dirty = d;
}

bool
RuleListView::HasSelection() const
{
	return (CurrentSelection()>=0);
}


const char*
RuleListView::RuleName(int index) const
{
	ITEM_INIT
	if(item) return item->Name();
	return NULL;
}
void
RuleListView::SetRuleName(const char* name, int index)
{
	ITEM_INIT
	if(item==NULL)return;
	item->SetName(name);
	InvalidateItem(index);
	is_dirty=true;
}

uint32
RuleListView::RuleData(int index) const
{
	ITEM_INIT
	if(item) return item->Rule();
	return -1;
}
void
RuleListView::SetRuleData(uint32 rule, int index)
{
	ITEM_INIT
	if(item==NULL)return;
	item->SetRule(rule);
	InvalidateItem(index);
	is_dirty=true;
}

void
RuleListView::RemoveRule()
{
	int i = CurrentSelection();
	if(i<0) return;
	RemoveRule(i);
	Select(i);
	ScrollToSelection();
}


void
RuleListView::RemoveRule(int index)
{
	BListItem* item = RemoveItem(index);
	if(item)
	{
		delete item;
		is_dirty=true;
	}
}


void
RuleListView::RemoveAll()
{
	int n = CountItems();
	for(int i=n-1;i>=0;i--) RemoveRule(i);
	is_dirty=(n>0);
}

void
RuleListView::AddRule(const char* name, uint32 rule)
{
	int i = CurrentSelection();
	AddRule(name, rule, i);
	Select(i);
	ScrollToSelection();
}


void
RuleListView::AddRule(const char* name, uint32 rule, int index)
{
	int N=CountItems();
	if(N>=MAX_ITEMS) return;
	if((index<0)||(index>N)) index = N;
	AddItem(new RuleItem(name, rule), index);
	is_dirty=true;
}


void
RuleListView::CopyRule()
{
	BAutolock l(clip_board_lock);
	if(l.IsLocked())
	{
		int index(-1);
		ITEM_INIT
		if(item) clip_board=*item;
	}
}

void
RuleListView::PasteRule()
{
	BAutolock l(clip_board_lock);
	if(l.IsLocked())
		AddRule(clip_board.Name(), clip_board.Rule());
}

extern status_t ReadString(BDataIO* f, BString* s, size_t limit);
extern status_t WriteString(BDataIO* f, BString* s);
extern bool ReadRule(const BString& source, int32* offset, uint32* rule, BString* name);
extern void WriteRule(BString* dest, uint32 rule, const char* name); 

static bool
FileAlert()
{
	BAlert* alert = new BAlert
		( "File Alert"
		, "This file may not be created by LifeHex Application. Are you sure you want to load from the file?"
		, "Open"
		, "Don't open"
	);
	return (alert->Go()==1);
}


status_t
RuleListView::LoadRules(BDataIO* in)
{
	BString s;
	status_t status = ReadString(in, &s, 0x10000);
	if(status<B_OK) return status;
	
	if(s.Compare(DATA_HEADER, 7))
	{
		if(FileAlert()) return B_OK;
	}

	BString name;
	uint32 rule;
	int32 offset=0;

	RemoveAll();
	while(ReadRule(s, &offset, &rule, &name))
	{
		AddRule(name.String(), rule,-1);
	}
	return B_NO_ERROR;
}

status_t
RuleListView::SaveRules(BDataIO* out)
{	
	BString s(DATA_HEADER);
	s<<"\n";
	const int L=CountItems();
	for(int i=0;i<L;i++)
	{
		WriteRule(&s, RuleData(i), RuleName(i));
	}
	return WriteString(out, &s);
}





void
RuleListView::DialogRename()
{
	if(!HasSelection()) return;
	BWindow* w = new NameInputDialog
		( BPoint(100,100)
		, new BMessage(M_RENAME)
		, this
		, "Rename rule as:"
		, RuleName());
	w->AddToSubset(this->Window());
	w->Show();
	beep();
}

void
RuleListView::DialogAdd(BMessage* msg)
{
	int32 rule;
	if(msg->FindInt32("rule", &rule)!=B_OK) return;
	
	BMessage* m = new BMessage(M_ADD);
	m->AddInt32("rule", rule);
	BWindow* w = new NameInputDialog
		( BPoint(100,100)
		, m
		, this
		, "Add current rule as:"
		, "new rule");
	w->AddToSubset(this->Window());
	w->Show();
}


void
RuleListView::AddRule(BMessage* msg)
{
	int insert_index(-1);

	if(msg->WasDropped())
	{
		insert_index = IndexOf(ConvertFromScreen(msg->DropPoint()));
		if(insert_index<0)insert_index = CountItems();
		if(msg->ReturnAddress()==BMessenger(this))
			RemoveRule();
	}
	int32 rule;
	char* name;
	if(msg->FindInt32("rule", &rule)==B_OK)
	{
		if(msg->FindString("name", &name)==B_OK)
			AddRule(name, rule, insert_index);
//		else
//			DialogAdd(msg);
	}
}

void
RuleListView::Rename(BMessage*msg)
{
	char* n;
	if(msg->FindString("name", &n)==B_OK)
		SetRuleName(n);
}

void
RuleListView::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
	case M_DIALOG_ADD:
		Messenger().SendMessage(MG_GET_RULE, this);
		break;
	case MG_GET_RULE:
		DialogAdd(msg);
		break;
	case M_ADD:
		AddRule(msg);
		break;
	case M_REMOVE:
		RemoveRule();
		break;
	case M_COPY:
		CopyRule();
		break;
	case M_CUT:
		CopyRule();
		RemoveRule();
		break;
	case M_PASTE:
		PasteRule();
		break;	
	case M_DIALOG_RENAME:
		DialogRename();
		break;
	case M_RENAME:
		Rename(msg);
		break;
	default:
		BListView::MessageReceived(msg);
	}
}


void
RuleListView::SelectionChanged()
{
	BListView::SelectionChanged();
	int32 r = RuleData();
	if(r!=-1)
	{
		BMessage m(MG_SET_RULE);
		m.AddInt32("rule",r);
		Messenger().SendMessage(&m);
	}
}


status_t
RuleListView::Invoke(BMessage* msg)
{
	DialogRename();
	return B_OK;
}


void
RuleListView::KeyDown(const char* chars, int32 n)
{
	if(n==1)
		switch(chars[0])
		{
		case B_BACKSPACE:
			RemoveRule();
			return;
		case B_SPACE:
			DeselectAll();
			return;
		};
	BListView::KeyDown(chars, n);
}

bool
RuleListView::InitiateDrag(BPoint p, int32 index, bool selected)
{
	if (!selected) return BListView::InitiateDrag(p, index, selected);
	
	BMessage msg (M_ADD);
	BRect rect = ItemFrame (index);
	msg.AddInt32("rule", RuleData());
	msg.AddString("name", RuleName());
	DragMessage (&msg, rect) ;
	return true ;
}

/*void
RuleListView::KeyDown(const char* chars, int32 n)
{
	if(n!=1) return;
	switch(chars[0])
	{
	case B_RETURN:
		DialogRename();
		break;
	case B_DELETE:
		RemoveRule();
		break;
	case B_SPACE:
		DeselectAll();
		break;
	case B_UP_ARROW:n=-1;
	case B_DOWN_ARROW:
		if(HasSelection())
			Select(CurrentSelection()+n);
		else
			Select(n==1? CountItems()-1:0);
		break;
	};
}
*/

