#include "LifeDaemon.h"

#include "LifeHex.h"
#include "LifeScreen.h"
#include "PixelMap.h"
#include "Renderer.h"


LifeDaemon::LifeDaemon()
	: Thread("daemon", B_NORMAL_PRIORITY)
	, life_hex(NULL)
	, renderer(NULL)
	, screen(NULL)
	
	, period(1)
	, exit(false)
	, rate_mode(RELATIVE)
	, speed(50)
	, is_stepping(false)
	, avoid_flash(true)
	, step_sem(-1)
	, latency_sum(1000*NUM_SAMPLES)
	, interval_sum(1000*NUM_SAMPLES)
{
	locker = new BLocker("life daemon");
	step_sem = create_sem(1, "life daemon step");
	
	for(int i =0;i<NUM_SAMPLES;i++)
	{
		latencies[i]=1000;
		intervals[i]=1000;
	}
}


LifeDaemon::~LifeDaemon()
{
	exit = true;
	if(step_sem>0) delete_sem(step_sem);
	delete locker;
	long	st;
	WaitForExit(&st);
}

bool
LifeDaemon::Lock()
{
	return locker->Lock();
}
void
LifeDaemon::Unlock()
{
	locker->Unlock();
}

bool
LifeDaemon::IsLocked()
{
	return locker->IsLocked();
}

void LifeDaemon::SetPeriod(int p)
{
	if(p<=0) p =1;
	if(p>30)p=30;
	period=p;
}

int LifeDaemon::Period()
{
	return period;
}
void LifeDaemon::SetSpeed(int r)
{
	if(r<0) r=0;
	if(r>100) r=100;
	speed=r;
}

int LifeDaemon::Speed()
{
	return speed;
}
void LifeDaemon::SetAvoidFlash(bool r)
{
	avoid_flash = r;
}

bool LifeDaemon::AvoidFlash()
{
	return avoid_flash;
}


void
LifeDaemon::Go()
{
	if(is_stepping)
		if(step_sem>0) release_sem(step_sem);
	is_stepping = false;
}

void
LifeDaemon::Step()
{
	if(step_sem<=0) return;
	if(is_stepping) release_sem(step_sem);
	else is_stepping = true;
}

void
LifeDaemon::GetFrameRates(float* min, float*current, float*max)
{
	float i = interval_sum;
	float l = latency_sum;
	float n = NUM_SAMPLES;
	float a_sec = 1000*1000;
	
	*min = 1;
	*current = (i<=1000*n)? 1000: a_sec/(i/n);
	*max = (l<=1000*n)? 1000: a_sec/(l/n);
}

status_t LifeDaemon::Run()
{
	static const bigtime_t A_SEC=1000*1000;
	
	int	index=0;
	bigtime_t prev, now;
	hex_rule current_rule;
	bool can_flash;
	
	prev = system_time();
	while(!exit)
	{
		prev = system_time();
		
		if(life_hex)
		{
			int p=period;
			if(avoid_flash&&can_flash)p=(p+1)/2*2;
			for(int i=0;i<p;i++)
			{
				BAutolock l(locker);
				if(!l.IsLocked()) return B_ERROR;
				if(life_hex)
					life_hex->Generate();
				else
					break;
			}
		}
		else
		{
			snooze(1000);
			continue;
		}
		
		{
			BAutolock l(locker);
			if(!l.IsLocked()) return B_ERROR;
			if( (life_hex!=NULL) && (renderer!=NULL) && (screen!=NULL))
			{
				can_flash = CanFlash(life_hex->Rule());
				if(screen->LockBuffer())
				{
					PixelMap dest_buffer = screen->GetBufferInfo();
					PixelMap source_buffer = life_hex->GetBufferInfo();
					renderer->Render(&source_buffer, &dest_buffer);
					screen->UnlockBuffer();
				}
				screen->SyncScreen();
			}
		}
		
		if(is_stepping)
		{
			if(step_sem>0) acquire_sem(step_sem);
			continue;
		}

		now = system_time();
		{/*latency*/
			bigtime_t t = now - prev;
			latency_sum += t;
			latency_sum -= latencies[index];
			latencies[index] = t;
		}
		{
			static const bigtime_t MAX_SNOOZE = A_SEC;
			bigtime_t latency = latency_sum/NUM_SAMPLES;
				uint32 max_rate = A_SEC*60/latency;//frame rates per one minute
				uint32 min_rate = 60;
				if(min_rate> max_rate/2) min_rate = max_rate/2;
				uint32 rate = (max_rate-min_rate)*speed/100+min_rate;
			bigtime_t snooze_time = A_SEC*60/rate - latency;
			if(snooze_time<=0)snooze_time=1;
			if(snooze_time>=MAX_SNOOZE) snooze_time = MAX_SNOOZE;
			snooze(snooze_time);
		}
		now = system_time();
		{/*interval*/
			bigtime_t t = now - prev;
			interval_sum += t;
			interval_sum -= intervals[index];
			intervals[index] = t;
		}
		(++index) %= NUM_SAMPLES;
	}
	return B_OK;
}
